"""Ariadne 内置的 Dispatcher"""


import asyncio
import contextlib

from graia.broadcast import Broadcast
from graia.broadcast.entities.dispatcher import BaseDispatcher as AbstractDispatcher
from graia.broadcast.entities.signatures import Force
from graia.broadcast.interfaces.dispatcher import DispatcherInterface

from .message.chain import MessageChain
from .message.element import Source
from .typing import generic_isinstance, generic_issubclass


class MessageChainDispatcher(AbstractDispatcher):
    """从 MessageEvent 提取 MessageChain 的 Dispatcher"""

    @staticmethod
    async def catch(interface: DispatcherInterface):
        from .event.message import ActiveMessage, MessageEvent

        if isinstance(interface.event, (MessageEvent, ActiveMessage)) and generic_issubclass(
            MessageChain, interface.annotation
        ):
            return interface.event.message_chain


class ContextDispatcher(AbstractDispatcher):
    """提取上下文的 Dispatcher"""

    @staticmethod
    async def catch(interface: DispatcherInterface):
        from .app import Ariadne

        if generic_isinstance(interface.event, interface.annotation):
            return interface.event

        if generic_issubclass(Broadcast, interface.annotation):
            return Ariadne.service.broadcast

        if generic_issubclass(asyncio.AbstractEventLoop, interface.annotation):
            return Ariadne.service.broadcast.loop

        if generic_issubclass(Ariadne, interface.annotation):
            return Ariadne.current()


class NoneDispatcher(AbstractDispatcher):
    """给 Optional[...] 提供 None 的 Dispatcher"""

    @staticmethod
    async def catch(interface: DispatcherInterface):
        if generic_isinstance(type(None), interface.annotation):
            return Force(None)


class SourceDispatcher(AbstractDispatcher):
    """提取 MessageEvent 消息链 Source 元素的 Dispatcher"""

    @staticmethod
    async def catch(interface: DispatcherInterface):
        from .event.message import ActiveMessage, MessageEvent

        if isinstance(interface.event, (MessageEvent, ActiveMessage)) and generic_issubclass(
            Source, interface.annotation
        ):
            return interface.event.message_chain.get_first(Source)


class SenderDispatcher(AbstractDispatcher):
    """
    从 MessageEvent 提取 sender 的 Dispatcher.
    """

    @staticmethod
    async def catch(interface: DispatcherInterface):
        from .event.message import MessageEvent

        if isinstance(interface.event, MessageEvent):
            with contextlib.suppress(TypeError):
                if generic_isinstance(interface.event.sender, interface.annotation):
                    return interface.event.sender


class SubjectDispatcher(AbstractDispatcher):
    """从 ActiveMessage 提取 subject 的 Dispatcher"""

    @staticmethod
    async def catch(interface: DispatcherInterface):
        from .event.message import ActiveMessage

        if isinstance(interface.event, ActiveMessage) and generic_issubclass(
            interface.annotation, interface.event.subject
        ):
            return interface.event.subject


class BaseDispatcher(AbstractDispatcher):
    """空 Dispatcher"""

    @staticmethod
    async def catch(*_):
        pass



from __future__ import annotations

import enum
import functools
import inspect
import re
from contextvars import ContextVar
from typing import (
    Any,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    List,
    MutableMapping,
    Optional,
    Set,
    Tuple,
    TypeVar,
    Union,
)
from weakref import WeakKeyDictionary, WeakSet

from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.entities.dispatcher import BaseDispatcher
from graia.broadcast.interfaces.dispatcher import DispatcherInterface
from typing_extensions import Self

from ...typing import MaybeFlag, Sentinel, T
from ..chain import MessageChain
from ..element import Element, Plain, Quote, Source

L_PAREN = ("{", "[")
R_PAREN = ("}", "]")
ESCAPE = {
    "\\": "\x00",
    "[": "\x01",
    "]": "\x02",
    "{": "\x03",
    "}": "\x04",
    "|": "\x05",
}
R_ESCAPE = {v: k for k, v in ESCAPE.items()}


def escape(string: str) -> str:
    """转义字符串

    Args:
        string (str): 要转义的字符串

    Returns:
        str: 转义后的字符串
    """
    for k, v in ESCAPE.items():
        string = string.replace("\\" + k, v)
    string = string.replace("\\", "")
    return string


def unescape(string: str) -> str:
    """逆转义字符串, 自动去除空白符

    Args:
        string (str): 要逆转义的字符串

    Returns:
        str: 逆转义后的字符串
    """
    for k, v in R_ESCAPE.items():
        string = string.replace(k, v)
    return string.strip()


class Text:
    __slots__ = "choice"
    choice: FrozenSet[str]

    def __init__(self, choice: Union[Iterable[str], str]) -> None:
        self.choice = frozenset((choice,)) if isinstance(choice, str) else frozenset(choice)

    def __repr__(self) -> str:
        return f"Text({self.choice!r})"


class Param:
    __slots__ = ("names",)
    names: FrozenSet[str]

    def __init__(self, names: Iterable[str]) -> None:
        self.names = frozenset(names)

    def __repr__(self) -> str:
        return f"Param({self.names!r})"


class AnnotatedParam:
    __slots__ = ("name", "annotation", "default", "wildcard")
    name: str
    annotation: Optional[str]
    default: Optional[str]
    wildcard: bool

    def __init__(
        self,
        name: str,
        wildcard: bool = False,
        annotation: Optional[str] = None,
        default: Optional[str] = None,
    ) -> None:
        self.name = name
        self.annotation = annotation
        self.default = default
        self.wildcard = wildcard

    def __repr__(self) -> str:
        return (
            f"AnnotatedParam({'...' if self.wildcard else ''}{self.name}: {self.annotation} = {self.default})"
        )

    def to_param(self) -> Param:
        return Param((self.name,))


U_Token = Union[Text, Param, AnnotatedParam]

ann_assign = re.compile(r"(?P<name>[^:=]+)(?P<annotation>:[^=]+)?(?P<default>=.+)?")


def parse_param(param_str: str) -> Union[Param, AnnotatedParam]:
    wildcard: bool = param_str.startswith("...")
    if wildcard:
        param_str = param_str[3:]
    match = ann_assign.match(param_str)
    assert match, f"Invalid param: {param_str}"
    names, *extra = match.groups()
    names = [unescape(name).strip() for name in names.split("|")]
    if not wildcard and not any(extra):
        return Param(names)
    assert len(names) == 1, f"Invalid param: {param_str}"
    return AnnotatedParam(
        names[0], wildcard, *map(lambda s: unescape(s).strip().lstrip(":=").strip() if s else None, extra)
    )


def _pop(char_stk: List[str]) -> str:
    piece = "".join(char_stk)
    char_stk.clear()
    return piece


def tokenize(string: str) -> List[U_Token]:
    """将字符串转义化, 并处理为 Text,  Param 两种 token

    Args:
        string (str): 要处理的字符串

    Returns:
        List[Tuple[CommandToken, List[int, str]]]: 处理后的 Token
    """

    string = escape(string)

    paren: str = ""
    char_stk: List[str] = []
    token: List[U_Token] = []
    pop = functools.partial(_pop, char_stk)

    for index, char in enumerate(string):
        if char in L_PAREN + R_PAREN:
            if char in L_PAREN:
                assert not paren, (
                    f"""Duplicated parenthesis character "{char}" @ {index} !"""
                    """Are you sure you've escaped with "\\"?"""
                )
                paren = char
            elif char in R_PAREN:
                piece = pop()
                assert paren, f"No matching parenthesis: {paren} @ {index}"
                if paren == "[":  # CHOICE
                    token.append(Text(unescape(x) for x in piece.split("|")))
                elif paren == "{":  # PARAM
                    token.append(parse_param(piece))
                paren = ""
        elif char == " " and not paren:
            if char_stk:
                token.append(Text(pop()))
        else:
            char_stk.append(char)

    assert not paren, f"Unclosed parenthesis: {paren}"

    if char_stk:
        token.append(Text(pop()))

    return token


class MatchEntry:
    def __init__(self, tokens: List[U_Token]) -> None:
        self.nodes: List[MaybeFlag[FrozenSet[str]]] = [
            token.choice if isinstance(token, Text) else Sentinel for token in tokens
        ]
        self.tokens: List[Union[Text, Param]] = [
            token.to_param() if isinstance(token, AnnotatedParam) else token for token in tokens
        ]
        self.params: List[Param] = [token for token in self.tokens if isinstance(token, Param)]


T_MatchEntry = TypeVar("T_MatchEntry", bound=MatchEntry)


class MatchNode(Generic[T_MatchEntry]):
    __slots__ = ("next", "entries")
    next: Dict[MaybeFlag[str], MatchNode[T_MatchEntry]]
    entries: WeakSet[T_MatchEntry]

    def __init__(self) -> None:
        self.next = {}
        self.entries = WeakSet()

    def copy(self) -> Self:
        new_obj = self.__class__()
        new_obj.next = self.next.copy()
        new_obj.entries = self.entries.copy()
        return new_obj

    def push(self, entry: T_MatchEntry, index: int = 0) -> None:
        if index >= len(entry.nodes):
            self.entries.add(entry)
        current: MaybeFlag[FrozenSet[str]] = entry.nodes[index]
        if current is Sentinel:
            self.next.setdefault(current, MatchNode()).push(entry, index + 1)
        else:
            target_nodes: List[MatchNode[T_MatchEntry]] = []
            conflicts: Dict[MatchNode[T_MatchEntry], Set[str]] = {}
            for piece, node in {piece: self.next[piece] for piece in current if piece in self.next}.items():
                conflicts.setdefault(node, set()).add(piece)
            for old_node, conflict_fields in conflicts.items():
                new_node = old_node.copy()
                target_nodes.append(new_node)
                for field in conflict_fields:
                    self.next[field] = new_node
                new_node.push(entry, index + 1)
                current -= conflict_fields
            if current:
                new_node = MatchNode()
                new_node.push(entry, index + 1)
                for field in current:
                    self.next[field] = new_node

    def _inspect(self, fwd=""):
        if not self.next:
            print(fwd)
        for k, node in self.next.items():
            node._inspect(f"{fwd}{'<PARAM>' if k is Sentinel else k} ")


class _RawEnum(enum.Enum):
    Raw = object()


raw = _RawEnum.Raw  # wildcard annotation object


def convert_empty(obj: T) -> MaybeFlag[T]:
    if obj is inspect.Parameter.empty:
        return Sentinel
    if isinstance(obj, Decorator):
        return Sentinel
    return obj


class ContextVarDispatcher(BaseDispatcher):
    """分发常量给指定名称的参数"""

    def __init__(self, data_ctx: ContextVar[Dict[str, Any]]) -> None:
        self.data_ctx = data_ctx

    async def catch(self, interface: DispatcherInterface):
        return self.data_ctx.get().get(interface.name)


q_split_cache: MutableMapping[MessageChain, Tuple[List[str], List[MessageChain]]] = WeakKeyDictionary()


def q_split(chain: MessageChain) -> Tuple[List[str], List[MessageChain]]:
    if chain in q_split_cache:
        return q_split_cache[chain]
    str_result: List[str] = []
    str_cache: List[str] = []
    result: List[MessageChain] = []
    chain_cache: List[Element] = []
    quote = ""

    for elem in chain.__root__:
        if elem.__class__ in (Quote, Source):
            continue
        if not isinstance(elem, Plain):
            chain_cache.append(elem)
            str_cache.append("\x01")
            continue
        string = elem.text
        cache: List[str] = []
        for index, char in enumerate(string):
            if char in {"'", '"'}:
                if not quote:
                    quote = char
                elif (
                    char == quote and index and string[index - 1] != "\\"
                ):  # is current quote, not transfigured
                    quote = ""
                else:
                    cache.append(char)
                    continue
            elif not quote and char == " ":
                tmp = "".join(cache)
                chain_cache.append(Plain(tmp))
                str_cache.append(tmp)
                result.append(MessageChain(chain_cache.copy(), inline=True))
                str_result.append("".join(str_cache))
                chain_cache.clear()
                cache.clear()
                str_cache.clear()
            elif char != "\\":
                cache.append(char)
        if cache:
            tmp = "".join(cache)
            chain_cache.append(Plain(tmp))
            str_cache.append(tmp)
    if chain_cache:
        result.append(MessageChain(chain_cache, inline=True))
        str_result.append("".join(str_cache))
    q_split_cache[chain] = (str_result, result)
    return str_result, result



"""Commander: 便捷的指令触发体系"""
import abc
import asyncio
import contextlib
import inspect
from contextvars import ContextVar
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Deque,
    Dict,
    FrozenSet,
    Iterable,
    List,
    Literal,
    NamedTuple,
    Optional,
    Sequence,
    Set,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from graia.broadcast import Broadcast, Listener
from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.entities.exectarget import ExecTarget
from graia.broadcast.exceptions import PropagationCancelled
from graia.broadcast.typing import T_Dispatcher
from graia.broadcast.utilles import dispatcher_mixin_handler
from pydantic import BaseConfig, BaseModel, ValidationError, create_model, validator
from pydantic.fields import ModelField
from typing_extensions import LiteralString

from ...context import event_ctx
from ...dispatcher import ContextDispatcher
from ...event.message import MessageEvent
from ...model import AriadneBaseModel
from ...typing import MaybeFlag, Sentinel, Wrapper
from ...util import (
    constant,
    gen_subclass,
    get_stack_namespace,
    resolve_dispatchers_mixin,
)
from ..chain import MessageChain
from ..element import Element
from .util import (
    AnnotatedParam,
    ContextVarDispatcher,
    MatchEntry,
    MatchNode,
    Param,
    Text,
    convert_empty,
    q_split,
    raw,
    tokenize,
)

T_Callable = TypeVar("T_Callable", bound=Callable)


def chain_validator(value: MessageChain, field: ModelField) -> Union[MessageChain, Element, str]:
    """
    MessageChain 处理函数.
    应用作 pydantic 的 Model validator.
    取决于字段类型标注, 若与消息链, 消息元素无关则会直接把消息链用 asDisplay 转换为字符串.

    Args:
        value (MessageChain): 消息链
        field (ModelField): 当前的 model 字段

    Returns:
        Union[MessageChain, Element, str]: 取决于字段类型标注
    """
    if field.outer_type_ is MessageChain:
        return value
    if issubclass(field.type_, Element):
        assert value.content[0].__class__ is field.type_
        return value.content[0]
    if isinstance(value, MessageChain):
        return str(value)
    if value is None:
        return field.default
    return value


class ParamDesc(abc.ABC):
    model: Optional[Type[BaseModel]]
    dest: Optional[str]
    default_factory: MaybeFlag[Callable[[], Any]]

    @abc.abstractmethod
    def gen_model(self, validators: Iterable[Callable]) -> None:
        """生成用于 pydantic 解析的 model 属性

        Args:
            validators (Iterable[Callable]): 用作 validator 的 Callable 可迭代对象
        """
        ...


class CommanderModelConfig(BaseConfig):
    copy_on_model_validation: bool = False


def make_model(name: str, validators: Iterable[Callable] = (), **fields) -> Type[BaseModel]:
    return create_model(
        name,
        __config__=CommanderModelConfig,
        __validators__={
            f"#validator_{i}#": validator("*", pre=True, allow_reuse=True)(v)
            for i, v in enumerate(validators)
        },
        **{k: (v, ...) for k, v in fields.items()},
    )


class Slot(ParamDesc):
    """Slot"""

    def __init__(
        self,
        target: Union[str, int],
        type: MaybeFlag[Type] = Sentinel,
        default: MaybeFlag[Any] = Sentinel,
        default_factory: MaybeFlag[Callable[[], Any]] = Sentinel,
    ) -> None:
        self.target = str(target)
        self.type: Union[Literal[Sentinel, raw], Type] = type
        if self.type == "raw":
            self.type = raw
        self.default_factory = constant(default) if default is not Sentinel else default_factory
        self.dest: Optional[str] = None
        self.model: Optional[Type[BaseModel]] = None

    def gen_model(self, validators: Iterable[Callable]) -> None:
        if self.model or self.type is raw:
            return
        if self.type is Sentinel:
            self.type = MessageChain
        self.model = make_model("SlotModel", validators, val=self.type)

    def merge(self, other: "Slot") -> None:
        if self.type is Sentinel and other.type is not Sentinel:
            self.type = other.type
        if self.default_factory is Sentinel and other.default_factory is not Sentinel:
            self.default_factory = other.default_factory


class Arg(ParamDesc):
    """Argument"""

    headers: FrozenSet[str]

    def __init__(
        self,
        pattern: str,
        type: MaybeFlag[Type] = Sentinel,
        default: MaybeFlag[Any] = Sentinel,
        default_factory: MaybeFlag[Callable[[], Any]] = Sentinel,
    ) -> None:

        self.tags: List[str] = []
        self.dest: Optional[str] = None
        self.type: MaybeFlag[Type] = Sentinel
        self.model: Optional[Type[BaseModel]] = None
        iter_tokens = iter(tokenize(pattern))
        headers = next(iter_tokens)
        assert isinstance(headers, Text), "Required argument pattern!"
        self.headers = headers.choice
        for token in iter_tokens:
            assert isinstance(token, Param), "Argument pattern can only be presented at header!"
            assert len(token.names) == 1, "Arg param cannot have alias!"
            self.tags.append(next(iter(token.names)))
        nargs = len(self.tags)
        if nargs == 0:
            self.type = bool
            self.default_factory = constant(False)
        elif nargs == 1:
            self.type = MessageChain
        if default is not Sentinel:
            self.default_factory = constant(default)
        elif default_factory is not Sentinel:
            self.default_factory = default_factory
        if type is not Sentinel:
            self.type = type

    def gen_model(self, validators: Iterable[Callable]) -> None:
        if self.model:
            return
        if (
            isinstance(self.type, type)
            and issubclass(self.type, BaseModel)
            and not issubclass(self.type, AriadneBaseModel)
        ):
            self.model = self.type
            return

        nargs = len(self.tags)
        if nargs == 0:  # Set default
            self.model = make_model("ArgModel", validators, val=self.type)
        elif nargs == 1:
            self.model = make_model("ArgModel", validators, **{self.tags[0]: self.type})
        if self.model is None:
            raise ValueError(f"You didn't supply a suitable model for {self.dest}!")

    def update(self, annotation: MaybeFlag[Any], default: MaybeFlag[Any]) -> None:
        if self.type is Sentinel and annotation is not Sentinel:
            self.type = annotation
        if self.default_factory is Sentinel and default is not Sentinel:
            self.default_factory = constant(default)

    def __repr__(self) -> str:
        return f"Arg([{'|'.join(self.headers)}]{''.join(f' {{{tag}}}' for tag in self.tags)}"


class CommandEntry(MatchEntry, ExecTarget):
    """命令信息的存储数据结构, 同时可作为 ExecTarget"""

    def __init__(self, priority: int) -> None:
        self.priority = priority
        self.slot_map: Dict[str, Slot] = {}
        self.arg_map: Dict[str, Arg] = {}
        self.targets: Set[str] = set()
        self.header_map: Dict[str, Arg] = {}
        self.extra: Optional[AnnotatedParam] = None
        self._arg_name_map: Optional[Dict[Arg, str]] = None
        self._slot_targets: Optional[Tuple[FrozenSet[str], ...]] = None

    @property
    def arg_name_map(self) -> Dict[Arg, str]:
        if not self._arg_name_map:
            self._arg_name_map = {v: k for k, v in self.arg_map.items()}
        return self._arg_name_map

    @property
    def slot_targets(self) -> Tuple[FrozenSet[str], ...]:
        if not self._slot_targets:
            self._slot_targets = tuple(
                frozenset(name for name in param.names if name in self.slot_map) for param in self.params
            )
        return self._slot_targets

    def compile_arg(self, compile_result: Dict[str, Any], arg_data: Dict[str, List[MessageChain]]) -> None:
        for arg in self.arg_name_map:
            if TYPE_CHECKING:
                assert arg.default_factory is not Sentinel
                assert arg.type is not Sentinel
                assert arg.dest
                assert arg.model
            value = arg.default_factory()
            if arg.tags:
                if arg.dest in arg_data:  # provided in arg_data
                    value = dict(zip(arg.tags, arg_data[arg.dest]))
                if isinstance(value, arg.type):  # compatible with arg.type
                    compile_result[arg.dest] = value
                    continue
                if not isinstance(value, dict):
                    value = dict(zip(arg.tags, value if isinstance(value, Sequence) else [value]))

                if not issubclass(arg.type, BaseModel) or issubclass(arg.type, AriadneBaseModel):
                    # an generated BaseModel, deconstruct
                    compile_result[arg.dest] = arg.model(**value).__dict__[arg.tags[0]]
                else:
                    # user provided model
                    compile_result[arg.dest] = arg.model(**value)

            else:  # probably a bool
                if arg.dest in arg_data:
                    value = not value
                compile_result[arg.dest] = arg.model(val=value).__dict__["val"]

    def compile_extra(self, slot: Slot, extra_list: List[MessageChain]) -> Any:
        if TYPE_CHECKING:
            assert self.extra is not None
            assert slot.model
            assert slot.default_factory is not Sentinel
        if self.extra.wildcard:  # Wildcard
            if slot.type is raw:
                return MessageChain([" "]).join(*extra_list)
            return tuple(slot.model(val=chain).__dict__["val"] for chain in extra_list)
        # Optional
        value = extra_list[0] if extra_list else slot.default_factory()
        return slot.model(val=value).__dict__["val"]

    def compile_param(
        self,
        slot_data: Dict[str, MessageChain],  # Slot.target -> MessageChain
        arg_data: Dict[str, List[MessageChain]],  # Arg.dest -> MessageChain
        extra_list: List[MessageChain],
    ) -> Dict[str, Any]:
        compile_result: Dict[str, Any] = {}
        for target, slot in self.slot_map.items():
            if TYPE_CHECKING:
                assert slot.dest
                assert slot.model
            if self.extra and self.extra.name == target:  # skip Wildcard and Optional
                compile_result[slot.dest] = self.compile_extra(slot, extra_list)
            else:
                compile_result[slot.dest] = slot.model(val=slot_data[target]).__dict__["val"]
        self.compile_arg(compile_result, arg_data)
        self.oplog.clear()
        return compile_result


class MismatchError(ValueError):
    """指令失配"""


class ParseData(NamedTuple):
    index: int
    node: MatchNode[CommandEntry]
    params: Tuple[MessageChain, ...]


commander_param_ctx = ContextVar("commander_param_ctx")

param_dispatcher = ContextVarDispatcher(commander_param_ctx)


class Commander:
    """便利的指令触发体系"""

    def __init__(self, broadcast: Broadcast, listen: bool = True):
        """
        Args:
            broadcast (Broadcast): 事件系统
            listen (bool): 是否监听消息事件
        """
        self.broadcast = broadcast
        self.validators: List[Callable] = [chain_validator]
        self.match_root: MatchNode[CommandEntry] = MatchNode()
        self.entries: Set[CommandEntry] = set()

        if listen:
            self.broadcast.listeners.append(
                Listener(
                    self.execute,
                    self.broadcast.getDefaultNamespace(),
                    list(gen_subclass(MessageEvent)),
                )
            )

    def __del__(self):
        self.broadcast.listeners = [i for i in self.broadcast.listeners if i.callable != self.execute]

    def add_type_cast(self, *caster: Callable):
        """添加类型验证器 (type caster / validator)"""
        self.validators = [*reversed(caster), *self.validators]

    @staticmethod
    def parse_command(command: LiteralString, entry: CommandEntry) -> None:
        """从传入的命令补充 entry 的信息

        Args:
            command (LiteralString): 命令
            entry (CommandEntry): 命令的 entry
        """
        tokenize_result: List[Union[Text, Param, AnnotatedParam]] = tokenize(command)
        for token in tokenize_result:
            if isinstance(token, Text):
                assert all(
                    pattern not in entry.header_map for pattern in token.choice
                ), f"{token} conflicts with an Arg object!"

            elif isinstance(token, AnnotatedParam):
                if token.wildcard or token.default:
                    assert token is tokenize_result[-1], "Not setting wildcard / optional on the last slot!"
                    entry.extra = token
                assert token.name not in entry.targets, "Duplicated parameter slot!"
                entry.targets.add(token.name)
                parsed_slot = Slot(
                    token.name,
                    eval(
                        token.annotation or "_sentinel",
                        *get_stack_namespace(2, {"raw": raw, "_sentinel": Sentinel}),
                    ),
                    eval(token.default or "_sentinel", *get_stack_namespace(2, {"_sentinel": Sentinel})),
                )
                parsed_slot.dest = token.name  # assuming that param_name is consistent
                entry.slot_map.setdefault(token.name, parsed_slot).merge(
                    parsed_slot
                )  # parsed slot < provided slot
            elif isinstance(token, Param):
                for name in token.names:
                    assert name not in entry.targets, "Duplicated parameter slot!"
                    entry.targets.add(name)
        MatchEntry.__init__(entry, tokenize_result)

    @staticmethod
    def update_from_func(entry: CommandEntry) -> None:
        """从 entry 的 callable 更新 entry 的信息

        Args:
            entry (CommandEntry): 命令的 entry
        """
        for name, parameter in inspect.signature(entry.callable).parameters.items():
            annotation = convert_empty(parameter.annotation)
            default = convert_empty(parameter.default)
            if name in entry.targets:
                parsed_slot = Slot(name, annotation, default)
                parsed_slot.dest = name  # assuming that name is consistent
                entry.slot_map.setdefault(name, parsed_slot).merge(parsed_slot)  # parsed slot < provided slot
            if name in entry.arg_map:
                entry.arg_map[name].update(annotation, default)
            if default is not Sentinel:
                last_token = entry.tokens[-1]
                assert isinstance(last_token, Param), "Expected Param, not Text!"
                assert (
                    entry.slot_map[name].target in last_token.names
                ), "Not setting wildcard / optional on the last slot!"

    def command(
        self,
        command: LiteralString,
        settings: Optional[Dict[str, Union[Slot, Arg]]] = None,
        dispatchers: Sequence[T_Dispatcher] = (),
        decorators: Sequence[Decorator] = (),
        priority: int = 16,
    ) -> Wrapper:
        """装饰一个命令处理函数

        Args:
            command (str): 要处理的命令
            setting (Dict[str, Union[Slot, Arg]], optional): 参数设置.
            dispatchers (Sequence[T_Dispatcher], optional): 可选的额外 Dispatcher 序列.
            decorators (Sequence[Decorator], optional): 可选的额外 Decorator 序列.

        Raises:
            ValueError: 命令格式错误

        Returns:
            Callable[[T_Callable], T_Callable]: 装饰器
        """

        entry = CommandEntry(priority)
        self.entries.add(entry)  # Add strong ref

        for name, val in (settings or {}).items():
            if isinstance(val, Slot):
                entry.slot_map[val.target] = val
            elif isinstance(val, Arg):
                for header in val.headers:
                    entry.header_map[header] = val
                entry.arg_map[name] = val
            else:
                raise TypeError(f"Unknown setting value: {name} - {val!r}")
            val.dest = name

        Commander.parse_command(command, entry)

        def wrapper(func: T_Callable) -> T_Callable:

            ExecTarget.__init__(
                entry,
                func,
                [
                    ContextDispatcher(),
                    *resolve_dispatchers_mixin(dispatchers),
                ],
                list(decorators),
            )
            Commander.update_from_func(entry)
            for slot in entry.slot_map.values():
                slot.gen_model(self.validators)
                assert slot.dest is not None, "Slot dest is None!"
            for arg in entry.arg_map.values():
                arg.gen_model(self.validators)
                assert arg.dest is not None, "Argument dest is not set!"
                assert arg.default_factory is not Sentinel, f"{arg}'s default factory is not set!"
            if entry.extra:
                entry.nodes.pop()  # the last optional / wildcard token should not be on the MatchGraph

            self.match_root.push(entry)
            return func

        return wrapper

    def parse_rest(
        self,
        index: int,
        frags: List[MessageChain],
        str_frags: List[str],
        params: Tuple[MessageChain, ...],
        entry: CommandEntry,
    ) -> Optional[Tuple[CommandEntry, dict]]:
        # walks down optional, wildcard and Arg
        # extract slot data based on entry
        slot_data: Dict[str, MessageChain] = {
            name: chain for targets, chain in zip(entry.slot_targets, params) for name in targets
        }
        # slam all the rest data inside extra_list
        extra_list: List[MessageChain] = []
        # store MessageChains assigned to Arg in arg_data
        arg_data: Dict[str, List[MessageChain]] = {}
        # index frags
        while index < len(frags):
            str_frag: str = str_frags[index]
            if str_frag in entry.header_map:
                arg = entry.header_map[str_frag]
                index += 1
                if arg.dest:
                    if arg.dest in arg_data:  # if the arg is already assigned
                        return
                    arg_data[arg.dest] = frags[index : index + len(arg.tags)]
                index += len(arg.tags)
                if index > len(frags):  # failed
                    return None
                continue
            else:
                extra_list.append(frags[index])
                index += 1
        if entry.extra:
            if not entry.extra.wildcard and len(extra_list) > 1:
                return None
        elif extra_list:
            return None
        return entry, entry.compile_param(slot_data, arg_data, extra_list)

    async def execute(self, chain: MessageChain):
        """触发 Commander.

        Args:
            chain (MessageChain): 触发的消息链
        """

        str_frags, chain_frags = q_split(chain)
        pending_exec: Dict[int, List[Tuple[CommandEntry, dict]]] = {}
        pending_next: Deque[ParseData] = Deque([ParseData(0, self.match_root, ())])

        dispatchers: List[T_Dispatcher] = [param_dispatcher]

        if event := event_ctx.get(None):
            dispatchers.extend(dispatcher_mixin_handler(event.Dispatcher))

        def push_pending(index: int, nxt: MatchNode[CommandEntry], params: Tuple[MessageChain, ...]):
            for entry in nxt.entries:
                with contextlib.suppress(ValidationError):
                    if res := self.parse_rest(index, chain_frags, str_frags, params, entry):
                        pending_exec.setdefault(res[0].priority, []).append(res)
            pending_next.append(ParseData(index, nxt, params))

        while pending_next:
            index, node, params = pending_next.popleft()
            if index >= len(str_frags):
                continue
            chain_frag: MessageChain = chain_frags[index]
            frag: str = str_frags[index]
            index += 1
            if frag in node.next:
                nxt = node.next[frag]
                push_pending(index, nxt, params)
            if Sentinel in node.next:
                nxt = node.next[Sentinel]
                params += (chain_frag,)
                push_pending(index, nxt, params)

        for _, execution in sorted(pending_exec.items()):
            tasks: List[asyncio.Task] = []
            for entry, param in execution:
                commander_param_ctx.set(param)
                tasks.append(self.broadcast.loop.create_task(self.broadcast.Executor(entry, dispatchers)))
            done, _ = await asyncio.wait(tasks)
            for task in done:
                if task.exception() and isinstance(task.exception(), PropagationCancelled):
                    return
